package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.biganto.visual.roompark.data.repository.api.room_park.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.db.requrey.model.EstateEntity
import com.biganto.visual.roompark.data.repository.db.requrey.model.PlanPresetEntity
import com.biganto.visual.roompark.data.repository.file.FileModule
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.data.repository.mapper.fromRawList
import com.biganto.visual.roompark.domain.contract.FlatPlanContract
import com.biganto.visual.roompark.domain.model.PlanPresetModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 20.04.2020.
 */


class PlanRepository @Inject constructor(
    private val local: ILocalStore,
    private val api: IRoomParkApi,
    private val db: IDb,
    private val file: FileModule
)
    : FlatPlanContract {


    private fun getPlanTypesApi(estateId: Int): Observable<List<PlanPresetEntity>> =
        api.getEstatePlanTypes(estateId)
            .doOnNext { Timber.d("raw0 $it") }
            .map { fromRawList(it, ::fromRaw) }
            .map {
                it.onEach { plan ->
                    val e = EstateEntity()
                    e.setId(estateId)
                    plan.estateId = e
                }.toList()
            }
            .doOnNext(db::blockingUpsert)
            .subscribeOn(Schedulers.io())


    override fun getPlanTypes(estateId: Int): Observable<List<PlanPresetModel>> =
        Observable.mergeDelayError(
            arrayListOf(getPlanTypesApi(estateId))
        ).map { l -> List(l.size) { fromEntity(l[it]) } }


    private fun getPlanApi(estateId: Int
                           , planId:Int
                           , furniture:Boolean? = null
                           , sizes:Boolean? = null
                           , walls:Boolean? = null
                           , electric:Boolean? = null) =
        api.getDirectPlan(estateId, planId,
            furniture?:false
            ,sizes?:false
            ,walls?:false
            ,electric?:false)
            .map{
                val sFile = getPlanFile(
                    estateId = estateId,
                    planId = planId,
                    furniture = furniture,
                    walls = walls,
                    sizes = sizes,
                    electric = electric)
                file.saveFileToDisk(sFile,it)
                sFile.path
            }
            .subscribeOn(Schedulers.io())


    private fun getPlanFile(featuresVariant: PlanFeaturesVariant)
            = getPlanFile(
        estateId = featuresVariant.estateId,
        planId = featuresVariant.planId,
        furniture = featuresVariant.furniture,
        sizes = featuresVariant.sizes,
        walls = featuresVariant.walls,
        electric = featuresVariant.electric
    )

    private fun getPlanFile(estateId: Int
                            , planId:Int
                            , furniture:Boolean? = null
                            , sizes:Boolean? = null
                            , walls:Boolean? = null
                            , electric:Boolean? = null)
            = FileModule.getDirectory(file.context
        , FileModule.FileDirectory.PlanTypeDir(
            estateId = estateId,
            planId = planId,
            furniture = furniture,
            walls = walls,
            sizes = sizes,
            electric = electric
        ))

//    override fun getEmptyPlan(estateId: Int
//                              ,planId:Int): Observable<String> =
//        Observable.mergeDelayError(
//            arrayListOf(getPlanApi(estateId,planId))
//        )

    override fun getPlan(estateId: Int
                         ,planId:Int
                         , furniture:Boolean?
                         , sizes:Boolean?
                         , walls:Boolean?
                         , electric:Boolean?): Observable<String> =
        Observable.fromCallable { getPlanFile(
            estateId = estateId,
            planId = planId,
            furniture = furniture,
            walls = walls,
            sizes = sizes,
            electric = electric) }.switchMap {
            if (it.exists()) Observable.just(it.path)
            else getPlanApi(estateId
                ,planId
                , furniture
                , sizes
                , walls
                , electric
            )
        }




//    fun getPlanRequestString(estateId: Int
//                                      , planId:Int
//                                      , furniture:Boolean
//                                      , sizes:Boolean
//                                      , walls:Boolean
//                                      , electric:Boolean
//    ) = api.getDirectPlan(estateId,planId,furniture,sizes,electric).

}


val PlanPresetModel.featuresVariants : List<PlanFeaturesVariant>
    get() {
        if (this.features.isNullOrEmpty()) return  arrayListOf()
        val variants = this.features.size
        return arrayListOf()
    } //List9  PlanFeaturesVariant()

data class PlanFeaturesVariant(
    val estateId: Int,
    val planId:Int,
    val furniture:Boolean?,
    val sizes:Boolean?,
    val walls:Boolean?,
    val electric:Boolean?
)